//-----------------------------------------------------------------------------------------
//------------------------------ SISTEMA DE DEFESA  MACRO-2 -------------------------------
//-----------------------------------------------------------------------------------------


#include <IRremote.h>
#include <Servo.h>

// Define o servo
#define SERVO 11
Servo s;
int pos = 80; //Posição inicial definida pela localização do servo no mecanismo final
int c = 0;

// Define o receptor
const int RECV_PIN = 7;
IRrecv irrecv(RECV_PIN);
decode_results results;

// Define os pinos do driver
const int STEP = 3;
const int DIR = 4;
const int ang = 0.1125; //O valor angular correspondente a cada passo

// Alvo
int alvo_ang;
int alvo_inc;
int x = 0;
boolean alvo = false;
boolean indicador = false;
boolean disparo = false;

//Disparo
int led = 2;

//Serial
const byte charsNumericos = 32;
char charsRecebidos[charsNumericos];
char charsTemporarios[charsNumericos];// Arranjo temporário para analisar o dado recebido
char estado;
boolean novo_dado = false;
boolean c1 = true;
boolean c2 = true;
boolean c3 = true;






//-----------------------------------------------------------------------------------------

void setup() {
  Serial.begin(9600);

  // Seta os pinos do driver como OUTPUT
  pinMode(STEP, OUTPUT);
  pinMode(DIR, OUTPUT);

  // Configurações do servo
  s.attach(SERVO);
  s.write(80);

  // Configurações do receptor
  irrecv.enableIRIn();
  irrecv.blink13(true);

  // Seta a rotacao para o sentido horario
  digitalWrite(DIR, HIGH);

  //Seta o LED como OUTPUT
  pinMode(led, OUTPUT);

  //Mensagem inicial no Monitor Serial;
  Serial.println("Sistema de Defesa - Detecção de alvos");
  Serial.println("Digite 1 para Manual e 2 para Automática: ");
  Serial.println();
}





//-----------------------------------------------------------------------------------------

void loop() {
  
  //Se o Monitor estiver disponível, salva o valor da opção escolhida na variavel estado
  while (Serial.available() > 0 && c1 == true) { 
    estado = Serial.read();
    c1 = false; //Garante que receba o valor uma única vez
  }

  //Executa a opcão escolhida 
  switch (estado) {

    case '1':
      manual();
      break;

    case '2':
      automatico();
      break;
  }

  //Após a execução inicia o protocolo de disparo
  atirar();
}





//-----------------------------------------------------------------------------------------

void automatico() {
  
  //Dados de Introdução da opção no Monitor Serial
  while (c3 == true) { 
    Serial.println("Modo automatico escolhido!");
    Serial.println();
    Serial.println("Escaneando...");
    c3 = false; //Garante que sejam exibidos apenas uma vez
  }


  //Executa os movimentos Pan e Tilt enquanto o alvo não foi encontrado
  while ( alvo == false ) { 

    passo(); //Move o motor de passo
    mostrarDados(); //Exibe os dados de posição
    if (irrecv.decode(&results)) { //Checa o receptor, caso receber algo salva nas variaveis do alvo
      irrecv.resume();
      alvo = true;
      alvo_ang = x;
      alvo_inc = pos;
      mostrarDadosAlvo();
    }

    //Após o mecanismo rotacionar 10 graus inicia a inclinação pelo servo motor
    if ( c == 89 ) {
      for (pos = 80; pos > 40; pos--) {
        s.write(pos); //Move o servo motor
        delay(50);
        mostrarDados(); //Exibe os dados de posição
        if (irrecv.decode(&results)) { //Checa o receptor, caso receber algo salva nas variaveis do alvo
          irrecv.resume();
          alvo = true;
          alvo_ang = x;
          alvo_inc = (80 - pos);
          mostrarDadosAlvo();
          pos = 40;
        }
      }
      if ( alvo == false) { //Se o movimento não encontrou o alvo, retorna o servo à posição inicial
        c = 0;
        s.write(80);
        delay(100);
        pos = 80;
      }
    }
    //Itera as variaveis de posição 
    x = x + 1; 
    c = c + 1;
    if ((x * ang) > 360) {
      x = 0;
    }
  }
  disparo = true; //Se o alvo for encontrado habilita o disparo
}

//-----------------------------------------------------------------------------------------

void manual() {

  //Dados de Introdução da opção no Monitor Serial
  while (c2 == true) { 
    Serial.println("Modo manual escolhido!");
    Serial.println();
    Serial.println("Entre com o valor do Ângulo e da Inclinação");
    Serial.println("Digite nesse formato - <180,45>");
    Serial.println();
    c2 = false; //Evita que sejam exibidos novamente
  }

  receberComMarcadores(); //Recebe os dados com os separadores iniciais e finais

  if (novo_dado == true) {  //Analisa e separa os dados nas suas devidas variaveis
    strcpy(charsTemporarios, charsRecebidos);  // Faz uma copia temporaria para proteger os dados, porque o strtok() usado no analisarDados() troca as virgulas para \0

    analisarDado();
    mostrarDadoAnalisado();
    novo_dado = false;
  }

  if (alvo == true) {   //Com os dados das variaveis atualizados, move o servo e o motor de passo para as devidas posições
    while (x < alvo_ang / ang) {
      while (pos > (80 - alvo_inc)) {
        s.write(pos);
        delay(100);
        mostrarDados();
        pos = pos - 1;
      }
      passo();
      mostrarDados();
      x = x + 1; 
    }
    disparo = true; //Se o alvo for encontrado habilita o disparo
  }
}

//-------------------------------------------------------------------------------------------

void passo() { //Produz um pulso no pino STEP do driver
  digitalWrite(STEP, HIGH);
  delayMicroseconds(1000);
  digitalWrite(STEP, LOW);
  delayMicroseconds(1000);
}

//-------------------------------------------------------------------------------------------

void mostrarDados() { //Exibe os dados das posições atuais
  Serial.print("Ângulo: ");
  Serial.print(x * ang);
  Serial.print(" Inclinação: ");
  Serial.println(80 - pos);
}

//-------------------------------------------------------------------------------------------

void mostrarDadosAlvo() { //Exibe os dados das posições do alvo
  Serial.print("Ângulo: ");
  Serial.print(alvo_ang * ang);
  Serial.print(" Inclinação: ");
  Serial.print(80 - pos);
  Serial.println("  Alvo achado!");
}

//-------------------------------------------------------------------------------------------

void atirar() { //Sequência de disparo
  while (alvo = true && disparo == true) { //Se o disparo estiver habilitado e o alvo encontrado inicia a sequência
    Serial.println();
    Serial.println("Pronto para atirar?");
    Serial.println();
    Serial.print("Efetuando disparo em...");
    delay(1000);
    Serial.print("3");
    delay(1000);
    Serial.print(",2");
    delay(1000);
    Serial.println(",1");
    delay(1000);
    Serial.println();
    Serial.println("-- ALVO ELIMINADO --");
    digitalWrite(led, HIGH);
    //Reinicia as variaveis do alvo
    alvo = false; 
    disparo = false;
    c1 = true;
  }
}

//-------------------------------------------------------------------------------------------

void analisarDado() {      // Separa o dado em suas partes
  char * strtokIndx; // Isso é usado pelo strtok() como um indice
  strtokIndx = strtok(charsTemporarios, ",");     // Recebe a primeira parte
  alvo_ang = atoi(strtokIndx);     // Converte para um inteiro
  strtokIndx = strtok(NULL, ",");
  alvo_inc = atoi(strtokIndx);// Converte para um inteiro
}

//-----------------------------------------------------------------------------------------

void mostrarDadoAnalisado() { // Exibe os valores recebidos, agora formatados
  Serial.print("Ângulo: ");
  Serial.println(alvo_ang);
  Serial.print("Inclinação: ");
  Serial.println(alvo_inc);
  Serial.println();
  alvo = true;
}


//-----------------------------------------------------------------------------------------

void receberComMarcadores() { //Procedimento para receber os dados do serial com os marcadores '<' e '>'
  static boolean recebEmProgresso = false;
  static byte ndx = 0;
  char marcadorInicial = '<';
  char marcadorFinal = '>';
  char rc;

  while (Serial.available() > 0 && novo_dado == false) { //Se o monitor estiver disponivel e nenhum dado previamente recebido
    rc = Serial.read();

    //Se o recebimento estiver ocorrendo armazena os caracteres recebidos em um array 
    if (recebEmProgresso == true) { 
      if (rc != marcadorFinal) { //Enquanto o caracter recebido for diferente do marcador final, vai adicionando-o ao array 
        charsRecebidos[ndx] = rc;
        ndx++;
        if (ndx >= charsNumericos) {
          ndx = charsNumericos - 1;
        }
      }

      else { 
        charsRecebidos[ndx] = '\0'; //Termina a string e o recebimento dos dados
        recebEmProgresso = false;
        ndx = 0;
        novo_dado = true;
      }
    }

    //Se o primeiro caracter for o marcador inicial, inicia o recebimento de dados
    else if (rc == marcadorInicial) {
      recebEmProgresso = true;
    }
  }
}

//-----------------------------------------------------------------------------------------
